// aio1_in_top: Handles the serial digital input from the A2D and also
// generates required signalling for the AD7823 (A2D chip)
   
// This module converts serial into parallel data and instantiates 'fir' module
// and outputs fir_data_out[7:0]
   
module aio1_in_a2d(
         clk_20,      // 50 Mhz (20 ns) clock
         ADOUT,       // Serial digital input
         reset,       // push button reset
         conv_start,  // Control signal for the A2D
         ADCLK,       // Clock for the A2D
         fir_data_out // Parallel FIR Filter output
         );
   
   input clk_20,reset;
   input ADOUT;
   output conv_start,ADCLK;
   output [7:0]fir_data_out;
   
   reg conv_start;
   wire clk_80;
   reg [1:0]count_clk;
   reg [5:0]count_fir_clk;
   reg [7:0]fir_data_in;
   wire [7:0]fir_data_out_wire;
   wire fir_clk;
   reg ADCLK_en;
   
   reg [5:0]count_data;
   integer count_powerup;
   reg powerup_done;
   
   // clk_80 genration from clk_20 :
      // clk_80 is an 80ns clock genrated from the 20ns board clock
      // (frequency divided by four)
      // reset_pulse_80 is one clk_80 cycle width after posedge of reset
   
   always @(posedge clk_20 or posedge reset) begin
       if(reset == 1'b1)begin
           count_clk[1:0] <= 0;
       end
       else begin
           count_clk[1:0] <= count_clk[1:0] + 1;
       end
   end

   assign clk_80 = ~count_clk[1];
   
   // fir_clk generation from clk_80
      // Each new set of 8-bit input data to the FIR ...
      //    is available every 64 clock cycles of clk_80
      // Therefore, frequency of fir_clk = clk_80/64
      // fir_clk = fir_clk & ~(reset)
      //    this is because we dont want any sampling of inputs during power-up   
   
   always @(posedge clk_80 or posedge reset)begin
       if(reset == 1'b1)begin
           count_fir_clk[5:0] <= 0;
       end
       else begin
           count_fir_clk[5:0] <= count_fir_clk[5:0] + 1;
       end
   end

   assign fir_clk = (~count_fir_clk[5]) & (~reset);
   
   // conv_start signalling
   
   always @(posedge clk_80 or posedge reset) begin
       if(reset == 1'b1)begin
           count_powerup<= 0;
           count_data   <= 0;
           powerup_done <= 0;
           ADCLK_en     <= 0;
           conv_start   <= 0;
       end 
       else begin

           ADCLK_en <= 0;
           
           // Power-up sequnce
           
           if (powerup_done != 1'b1) begin
               if(count_powerup  <= 10)begin
                   conv_start    <= 1'b0;
                   count_powerup <= count_powerup + 1;
               end
               else if(count_powerup <= 30)begin
                   conv_start    <= 1'b1;
                   count_powerup <= count_powerup + 1;
               end
               else if (count_powerup <= 42) begin
                   conv_start    <= 1'b0;
                   count_powerup <= count_powerup + 1;
               end
               else begin
                   conv_start    <= 1'b1;
                   powerup_done  <= 1'b1;
               end
           end
           
           // if powerup is done (powerup_done == 1) then continue data sequence
           
           else begin
               
               count_data <= count_data + 1;
               
               if(count_data <= 3)begin
                   ADCLK_en <= 0;
                   conv_start <= 1'b1;
               end
               else if(count_data <= 4)begin
                   ADCLK_en <= 0;
                   conv_start <= 1'b0;
               end
               else if(count_data < 53)begin
                   ADCLK_en <= 0;
                   conv_start <= 1'b1;
               end
	       else if(count_data == 53)begin
  		   ADCLK_en <= 1;
		   conv_start <= 1'b1;
	       end
               
               // After the conversion has finished, enable ADCLK
               // Start taking in each bit of the serial data and assign it to data[7:0]
               
               else if(count_data == 54)begin
                   ADCLK_en <= 1;
                   conv_start <= 1'b1;
                   fir_data_in[7] <= ADOUT;
               end
               else if(count_data == 55)begin
                   ADCLK_en <= 1;
                   conv_start <= 1'b1;
                   fir_data_in[6] <= ADOUT;
               end
               else if(count_data == 56)begin
                   ADCLK_en <= 1;
                   conv_start <= 1'b1;
                   fir_data_in[5] <= ADOUT;
               end
               else if(count_data == 57)begin
                   ADCLK_en <= 1;
                   conv_start <= 1'b1;
                   fir_data_in[4] <= ADOUT;
               end
               else if(count_data == 58)begin
                   ADCLK_en <= 1;
                   conv_start <= 1'b1;
                   fir_data_in[3] <= ADOUT;
               end
               else if(count_data == 59)begin
                   ADCLK_en <= 1;
                   conv_start <= 1'b1;
                   fir_data_in[2] <= ADOUT;
               end
               else if(count_data == 60)begin
                   ADCLK_en <= 1;
                   conv_start <= 1'b1;
                   fir_data_in[1] <= ADOUT;
               end
               else if(count_data == 61)begin
                   ADCLK_en <= 1;
                   conv_start <= 1'b1;
                   fir_data_in[0] <= ADOUT;
               end
               
               // Disable the ADCLK after the 8-bit data transfer is done
               
               else begin
                   ADCLK_en <= 0;
                   conv_start <= 1'b1;
               end
           end
       end
   end
   
   assign ADCLK = ADCLK_en & ~clk_80;
   
   // Instantiate the 'fir' module with the obtained fir_data_in:
   
   fir f1(fir_clk, reset, (fir_data_in - 8'd128), fir_data_out_wire);
   
   assign fir_data_out = fir_data_out_wire + 8'd128;
   
   endmodule
   
